library(shiny)

red <- paste("<font color=\"red\">")
blue <- paste("<font color=\"blue\">")
green <- paste("<font color=\"green\">")
orange <- paste("<font color=\"orange\">")
black <- paste("<font color=\"black\">")

mycols <- c("red","blue","green","orange","black")

shinyServer(function(input, output) {

    npop <- 200
    x <- runif(npop,0,1)
    y <- runif(npop,0.55,1)
    x0 <- x
        
    mgroups <- reactive({
        if(input$which=="ACME") m <- 2
        if(input$which=="Undergraduates") m <- 4
        if(input$which=="Balls") m <- as.numeric(input$ngroups)
        m
    })

    Id <- reactive({
        sample(1:mgroups(),size=npop,replace=T)        
    })
    
    samples <- reactive({    
       n <- as.numeric(input$nsample)
       whichcluster <- sample(1:6,2)
       if(input$method=="Simple Random Sample") smpls <- sample(1:npop,size=n)
       if(input$method=="Stratified Sampling") {
            smpls <- NULL
            m <- mgroups()
            k <- floor(n/m)
            j <- n-m*k
            for(i in 1:m) {
                smpls <- c(smpls,sample(c(1:npop)[Id()==i],size=k+ifelse(j>0,1,0)))  
                j <- j-1
            }    
       }                       
       if(input$method=="Systematic Sampling") {            
            last <- npop/n
            start <- sample(1:last,1)
            smpls <- start+c(0:(n-1))*last  
       }
       if(input$method=="Cluster Sampling") {       
          index <- list(1,1)
          for(i in 1:2) {
            if(whichcluster[i]==1) index[[i]] <- c(1:npop)[x<1/3 & y<0.8]  
            if(whichcluster[i]==2) index[[i]] <- c(1:npop)[x>1/3 & x<2/3 & y<0.8]
            if(whichcluster[i]==3) index[[i]] <- c(1:npop)[x>2/3 & y<0.8]
            if(whichcluster[i]==4) index[[i]] <- c(1:npop)[x<1/3& y>0.8]  
            if(whichcluster[i]==5) index[[i]] <- c(1:npop)[x>1/3 & x<2/3 & y>0.8]
            if(whichcluster[i]==6) index[[i]] <- c(1:npop)[x>2/3 & y>0.8]
          }
          k <- floor(n/2)
          j <- n-2*k
          smpls <- c(sample(index[[1]],size=k+j),sample(index[[2]],size=k))
       }   
       list(smpls,whichcluster,start)
       
    })
    
    data1 <- reactive({
        if(input$method=="Stratified Sampling") {
          m <- mgroups()
          for(i in 1:m)       
              x[Id()==i] <- (10-input$step1)/10*x0[Id()==i] + input$step1/10*((i-1)/m+1/m*x0[Id()==i])
        }      
        x
   })     

    data2 <- reactive({
        y0 <- y[samples()[[1]]]
        y1 <- (y0-0.6)/0.8
        y2 <- y0*(10-input$step2)/10+y1*input$step2/10
        y3 <- y
        y3[samples()[[1]]] <- y2
        y3
   })          
    
  
   output$text2 <- renderText({
      if(input$which=="ACME" )
          txt <- "<h4>Our population consists of men and women<p>
                We want to randomly select some people</h4>"
      if(input$which=="Undergraduates")          
          txt <- paste("<h4>Our population consists of undergraduate students,
                  identified as",red,"freshman (o),",blue," sophomores (&square;), ",
                  green,"juniors (&Delta;)",black," and",orange,"seniors &nabla;)",black,".<p>
                We want to randomly select some students</h4>")
      if(input$which=="Balls") {
          if(mgroups()==2)      
              txt <- paste("<h4>Our population consists of balls, each ball is either
                    red or blue<p>We want to randomly select some balls</h4>")
          if(mgroups()==3)      
              txt <- paste("<h4>Our population consists of balls, each ball is either
                    red, blue or green<p>We want to randomly select some balls</h4>")
          if(mgroups()==4)      
              txt <- paste("<h4>Our population consists of balls, each ball is either
                    red, blue, green or orange<p>We want to randomly select some balls</h4>")
          if(mgroups()==5)      
              txt <- paste("<h4>Our population consists of balls, each ball is either
                    red, blue, green, orange or black<p>We want to randomly select some balls</h4>")
                        
      }          
      txt
   })
    
   output$text1 <- renderText({
      n <- as.numeric(input$nsample)
      if(input$which=="ACME" ) {  
        if(input$method=="Simple Random Sample") 
          txt <- paste("<h4>Simple Random Sample: picks any of the people in the population</h4>")
        if(input$method=="Stratified Sampling") 
          txt <- paste("<h4>Stratified Sampling: divides people by gender, then selects from each gender</h4>")
        if(input$method=="Systematic Sampling") {
            txt <- c(paste("<h4>Systematic Sampling: labels each person from 1 to ",npop," , 
                chooses first person at random and then systematically picks every",
                round(npop/n),"<sup>th</sup> person</h4>"),"<h4>&nbsp;</h4>")
            if(input$step2>0) 
               txt[2] <- paste("<h4>First person chosen was person #",samples()[[3]],"</h4>")
        }       
        if(input$method=="Cluster Sampling") 
          txt <- paste("<h4>Cluster Sampling: devides people by location, randomly chooses location 
                and randomly chooses people from that location</h4>")
     }
     if(input$which=="Undergraduates" ) {  
        if(input$method=="Simple Random Sample") 
          txt <- paste("<h4>Simple Random Sample: picks any of the students in the population</h4>")
        if(input$method=="Stratified Sampling") 
          txt <- paste("<h4>Stratified Sampling: divides people by year, then selects from each year</h4>")
        if(input$method=="Systematic Sampling") {
            txt <- c(paste("<h4>Systematic Sampling: labels each student from 1 to ",npop," , 
                chooses first student at random and then systematically picks every",
                round(npop/as.numeric(input$nsample)),"<sup>th</sup> student</h4>"),"<h4>&nbsp;</h4>")
            if(input$step2>0) 
               txt[2] <- paste("<h4>First student chosen was #",samples()[[3]],"</h4>")
        }       
        if(input$method=="Cluster Sampling") 
          txt <- paste("<h4>Cluster Sampling: devides students by location, randomly chooses location 
                and randomly chooses student from that location</h4>")
     }
     if(input$which=="Balls" ) {  
        if(input$method=="Simple Random Sample") 
          txt <- paste("<h4>Simple Random Sample: picks any of the balls in the population</h4>")
        if(input$method=="Stratified Sampling") 
          txt <- paste("<h4>Stratified Sampling: divides balls by color, then selects from each color</h4>")
        if(input$method=="Systematic Sampling") {
            txt <- c(paste("<h4>Systematic Sampling: labels each ball from 1 to ",npop," , 
                chooses first ball at random and then systematically picks every",
                round(npop/as.numeric(input$nsample)),"<sup>th</sup> ball</h4>"),"<h4>&nbsp;</h4>")
            if(input$step2>0) 
               txt[2] <- paste("<h4>First ball chosen was #",samples()[[3]],"</h4>")
        }       
        if(input$method=="Cluster Sampling") 
          txt <- paste("<h4>Cluster Sampling: devides balls by location, randomly chooses location 
                and randomly chooses balls from that location</h4>")
     }     
     txt
   }) 
    
   output$plot <- renderPlot({          
        plot(c(-0.1, 1.1), c(-0.1, 1.1), axes = F, xlab = "", ylab = "", type = "n")
        mtext("Population",3,cex=2)
        
        if(input$which=="ACME") mysymbols <- c("W","M")
        if(input$which=="Undergraduates") mysymbols <- c(21,22,24,25)    
        if(input$which=="Balls") mysymbols <- rep(20,5)
        points(data1(),y,pch=mysymbols[Id()],col=mycols[Id()])
        rect(-0.1,0.505,1.1,1.1,lwd=3)
        if(input$method=="Stratified Sampling") {        
            if(input$step1==10) {
                m <- mgroups()
                xloc <- (1:(m-1))/m
                segments(xloc,0.52,xloc,1,lty=2)                
            }   
        }        
        if(input$method=="Systematic Sampling") 
            text(data1(),y+0.05,1:npop,cex=0.8)
        if(input$method=="Cluster Sampling") {
            segments(1/3,0.505,1/3,1.1,lty=2)
            segments(2/3,0.505,2/3,1.1,lty=2)
            segments(-0.1,0.8,1.1,0.8,lty=2)
            if(input$step2>0) {
              j <- samples()[[2]]
              for(i  in 1:2) {
                if(j[i]==1) rect(-0.1,0.505,1/3,0.8,density=3)
                if(j[i]==2) rect(1/3,0.505,2/3,0.8,density=3)
                if(j[i]==3) rect(2/3,0.505,1.1,0.8,density=3)
                if(j[i]==4) rect(-0.1,0.8,1/3,1.1,density=3)
                if(j[i]==5) rect(1/3,0.8,2/3,1.1,density=3)
                if(j[i]==6) rect(2/3,0.8,1.1,1.1,density=3)
                
              }
            }  
        }    
        if(input$step2>0) {
            points(data1(),data2(),pch=mysymbols[Id()],col=mycols[Id()])
            if(input$method=="Systematic Sampling") 
                  text(x[samples()[[1]]],data2()[samples()[[1]]]+0.05,samples()[[1]],cex=0.8)
            rect(-0.1,-0.1,1.1,0.495,lwd=3)
            text(0.5,0.45,"Sample",cex=2)               
        }
   })
           
  output$story <- renderText({ 
     index <- samples()[[1]]
     n <- input$nsample
     n1 <- floor(n/2)
     abt <- ifelse(2*n1==n,"","about") 
     if(input$which=="ACME") {   
        line <- paste("<h4>ACME company has",npop,"employees. They are located in 6 different 
            buildings in the city. Each employee has an id number, from 1 - ",npop,"</h4>")
        line[2] <- paste("<h4>They are planning on giving a questionaire to all their employees.
            First, though, they want to make sure that the questions are easily understood by everyone</h4>")
        line[3] <- paste("<h4>So they decide to do a small pilot study. For this they want to randomly
             select",n,"employees.</h4>")     
        if(input$method=="Simple Random Sample") {
            line[4] <- paste("<h4>They use a computer to randomly pick",input$nsample," numbers from 1 - "
                 ,npop,", and select the people with the corresponding id numbers.<p>
                 They pick</h4>")
            z <- paste(sort(index),collapse=" ")
            line[5] <- paste("<h4>",z,"</h4>",collapse="")
        }
        if(input$method=="Stratified Sampling") {
            line[4] <- paste("<h4>They decide that they should have equally many men and women in their sample, so
              they use a computer to randomly pick",abt,round(n/2)," numbers from 1 - "
                 ,npop," which correspond to females and the same for the males.<p>
                 They pick</h4>")     
                 id <- Id()[index]
                 z0 <- paste(sort(index[id==1]),collapse=" ")
                 z1 <- paste(sort(index[id==2]),collapse=" ")
                 
            line[5] <- paste("<h4>Females:",z0,"<p>Males:",z1,"</h4>")
        }
        if(input$method=="Systematic Sampling") {
            line[4] <- paste("<h4>They use a computer to randomly pick a number between 1 and",floor(npop/n)," 
                  and then select every",floor(npop/n),"<sup>th</sup> person after that.<p>
                 They pick the number",samples()[[3]],", and so their sample is</h4>")
            z <- paste(sort(index),collapse=" ")
            line[5] <- paste("<h4>",z,"</h4>",collapse="")
        }
        if(input$method=="Cluster Sampling") {
            line[4] <- paste("<h4>They don't want to have to go to each of the six buildings, so they decide to randomly
              choose two of the buildings and then randomly choose ",abt,floor(n/2),"employees in each building.<p>
                 They pick buildings",samples()[[2]][1],"and",samples()[[2]][2],"and then</h4>")     
                 z0 <- paste(sort(index[1:n1]),collapse=" ")
                 z1 <- paste(sort(index[(n1+1):n]),collapse=" ")
                 
            line[5] <- paste("<h4>Building",samples()[[2]][1],":",z0,"
                      <p>Building",samples()[[2]][2],":",z1,"</h4>")
        }        
     }   
     if(input$which=="Undergraduates") {   
        line <- paste("<h4>A certain course at some University as",npop,"students enrolled. 
            They are in one of six sections and come from each year, freshman to senior. Each student has an id number, from 1 - ",npop,"</h4>")
        line[2] <- paste("<h4>The University is planning on giving a questionaire to all the students in this course.
            First, though, they want to make sure that the questions are easily understood by everyone</h4>")
        line[3] <- paste("<h4>So they decide to do a small pilot study. For this they want to randomly
             select",n,"students.</h4>")     
        if(input$method=="Simple Random Sample") {
            line[4] <- paste("<h4>They use a computer to randomly pick",input$nsample," numbers from 1 - "
                 ,npop,", and select the students with the corresponding id numbers.<p>
                 They pick</h4>")
            z <- paste(sort(index),collapse=" ")
            line[5] <- paste("<h4>",z,"</h4>",collapse="")
        }
        if(input$method=="Stratified Sampling") {
            line[4] <- paste("<h4>They decide that they should have equally many students from each year in their sample, so
              they use a computer to randomly pick about",floor(n/4)," numbers from 1 - "
                 ,npop," which correspond to each year.<p>
                 They pick</h4>")     
                 id <- Id()[index]
                 z1 <- paste(sort(index[id==1]),collapse=" ")
                 z2 <- paste(sort(index[id==2]),collapse=" ")
                 z3 <- paste(sort(index[id==3]),collapse=" ")
                 z4 <- paste(sort(index[id==4]),collapse=" ")
                 
            line[5] <- paste("<h4>Freshman:",z1,"<p>Sophomores:",z2,
                          "<p>Juniors :",z3,"<p>Seniors :",z4,"</h4>")
                                  
        }
        if(input$method=="Systematic Sampling") {
            line[4] <- paste("<h4>They use a computer to randomly pick a number between 1 and",floor(npop/n)," 
                  and then select every",floor(npop/n),"<sup>th</sup> student after that.<p>
                 They pick the number",samples()[[3]],", and so their sample is</h4>")
            z <- paste(sort(index),collapse=" ")
            line[5] <- paste("<h4>",z,"</h4>",collapse="")
        }
        if(input$method=="Cluster Sampling") {
            line[4] <- paste("<h4>They don't want to have to go to each of the six sections, so they decide to randomly
              choose two of the sections and then randomly choose",abt,floor(n/2),"students in each of those two sections.<p>
                 They pick sections",samples()[[2]][1],"and",samples()[[2]][2],"and then</h4>")     
                 z0 <- paste(sort(index[1:n1]),collapse=" ")
                 z1 <- paste(sort(index[(n1+1):n]),collapse=" ")
                 
            line[5] <- paste("<h4>Section",samples()[[2]][1],":",z0,"
                      <p>Section",samples()[[2]][2],":",z1,"</h4>")
        }        
     }   
     if(input$which=="Balls") return("<h4>No story</h4>")   
         
     line[6] <- paste("<h4>They have done a ",red,input$method,"</h4>")
     line      
                 
  })

   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#sampling\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 1/16/2015</h6>")
      
  })   
    
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates various sampling methods
        <p>Click on Start Over! before using a different sampling method</h5>"
        
  })

})

